package cn.com.qmth.examcloud.core.examwork.dao.entity;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;

import org.springframework.format.annotation.DateTimeFormat;

import cn.com.qmth.examcloud.api.commons.enums.ExamSpecialSettingsType;
import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.web.jpa.JpaEntity;

/**
 * Created by songyue on 17/1/13.
 */
@Entity
@Table(name = "EC_E_EXAM", indexes = {
		@Index(name = "IDX_E_E_001001", columnList = "rootOrgId,name", unique = true),
		@Index(name = "IDX_E_E_001002", columnList = "rootOrgId,code", unique = true)})
public class ExamEntity extends JpaEntity {

	private static final long serialVersionUID = 4009839764353162256L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	/**
	 * 顶级机构Id
	 */
	@Column(nullable = false)
	private Long rootOrgId;

	/**
	 * 考试批次开始时间
	 */
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date beginTime;

	/**
	 * 考试批次结束时间
	 */
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date endTime;

	/**
	 * 考试名称
	 */
	@Column(nullable = false)
	private String name;

	/**
	 * 考试编码
	 */
	@Column(nullable = false)
	private String code;

	/**
	 * 考试类型
	 */
	@Column(nullable = false)
	@Enumerated(EnumType.STRING)
	private ExamType examType;

	/**
	 * 考试时长
	 */
	private Integer duration;

	@Column(nullable = false)
	private Boolean enable;

	/**
	 * 考试备注
	 */
	private String remark;

	/**
	 * 考试次数
	 */
	private Long examTimes;

	/**
	 * 考生锁定(为true时,不能更新考生)
	 */
	private Boolean examStudentLocked;

	/**
	 * 是否禁止考试
	 */
	private Boolean examLimit;

	/**
	 * 开启特殊设置
	 */
	@Column(nullable = false)
	private Boolean specialSettingsEnabled;

	/**
	 * 特殊设置类型
	 */
	@Column(nullable = true)
	@Enumerated(EnumType.STRING)
	private ExamSpecialSettingsType specialSettingsType;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Long getRootOrgId() {
		return rootOrgId;
	}

	public void setRootOrgId(Long rootOrgId) {
		this.rootOrgId = rootOrgId;
	}

	public Date getBeginTime() {
		return beginTime;
	}

	public void setBeginTime(Date beginTime) {
		this.beginTime = beginTime;
	}

	public Date getEndTime() {
		return endTime;
	}

	public void setEndTime(Date endTime) {
		this.endTime = endTime;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public ExamType getExamType() {
		return examType;
	}

	public void setExamType(ExamType examType) {
		this.examType = examType;
	}

	public Integer getDuration() {
		return duration;
	}

	public void setDuration(Integer duration) {
		this.duration = duration;
	}

	public Boolean getEnable() {
		return enable;
	}

	public void setEnable(Boolean enable) {
		this.enable = enable;
	}

	public String getRemark() {
		return remark;
	}

	public void setRemark(String remark) {
		this.remark = remark;
	}

	public Long getExamTimes() {
		return examTimes;
	}

	public void setExamTimes(Long examTimes) {
		this.examTimes = examTimes;
	}

	public Boolean getExamStudentLocked() {
		return examStudentLocked;
	}

	public void setExamStudentLocked(Boolean examStudentLocked) {
		this.examStudentLocked = examStudentLocked;
	}

	public Boolean getExamLimit() {
		return examLimit;
	}

	public void setExamLimit(Boolean examLimit) {
		this.examLimit = examLimit;
	}

	public Boolean getSpecialSettingsEnabled() {
		return specialSettingsEnabled;
	}

	public void setSpecialSettingsEnabled(Boolean specialSettingsEnabled) {
		this.specialSettingsEnabled = specialSettingsEnabled;
	}

	public ExamSpecialSettingsType getSpecialSettingsType() {
		return specialSettingsType;
	}

	public void setSpecialSettingsType(ExamSpecialSettingsType specialSettingsType) {
		this.specialSettingsType = specialSettingsType;
	}

}
