package cn.com.qmth.examcloud.core.examwork.service.cache;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamSpecialSettingsRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamSpecialSettingsEntity;
import cn.com.qmth.examcloud.support.cache.bean.ExamOrgSettingsCacheBean;
import cn.com.qmth.examcloud.web.cache.RandomObjectRedisCache;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;

@Service
public class ExamOrgSettingsCache extends RandomObjectRedisCache<ExamOrgSettingsCacheBean> {

	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamSpecialSettingsRepo examSpecialSettingsRepo;

	@Override
	public ExamOrgSettingsCacheBean loadFromResource(Object... keys) {
		Long examId = (Long) keys[0];
		Long orgId = (Long) keys[1];

		ExamEntity exam = GlobalHelper.getEntity(examRepo, examId, ExamEntity.class);

		if (null == exam) {
			throw new StatusException("002005", "考试不存在");
		}

		ExamOrgSettingsCacheBean bean = new ExamOrgSettingsCacheBean();

		bean.setId(exam.getId());
		bean.setExamType(exam.getExamType().name());
		bean.setCode(exam.getCode());

		ExamSpecialSettingsEntity examOrgEntity = examSpecialSettingsRepo
				.findByExamIdAndOrgIdAndCourseIdIsNullAndStudentIdIsNull(exam.getId(), orgId);
		if (null != examOrgEntity) {
			if (null != examOrgEntity.getBeginTime()) {
				bean.setBeginTime(examOrgEntity.getBeginTime());
			}
			if (null != examOrgEntity.getEndTime()) {
				bean.setEndTime(examOrgEntity.getEndTime());
			}
			if (null != examOrgEntity.getExamLimit()) {
				bean.setExamLimit(examOrgEntity.getExamLimit());
			}
		} else {
			bean.setHasValue(false);
		}

		return bean;
	}

	@Override
	protected String getKeyPrefix() {
		return "E_EXAM_ORG_SETTINGS:";
	}

	@Override
	protected int getTimeout() {
		// 2分钟
		return 60 * 2;
	}

}
