package cn.com.qmth.examcloud.core.examwork.service.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.api.commons.enums.CURD;
import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.DynamicEnum;
import cn.com.qmth.examcloud.commons.helpers.DynamicEnumManager;
import cn.com.qmth.examcloud.commons.helpers.poi.ExcelReader;
import cn.com.qmth.examcloud.commons.util.BooleanUtil;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.commons.util.DateUtil.DatePatterns;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.examwork.base.enums.ExamProperty;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamPropertyRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamSpecialSettingsRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamPropertyEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamSpecialSettingsEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamService;
import cn.com.qmth.examcloud.core.examwork.service.ExamStudentService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamSpecialSettingsInfo;
import cn.com.qmth.examcloud.core.examwork.service.cache.ExamPropertyCache;
import cn.com.qmth.examcloud.core.examwork.service.cache.ExamSettingsCache;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.StudentCacheBean;
import cn.com.qmth.examcloud.support.privilege.PrivilegeDefine;
import cn.com.qmth.examcloud.support.privilege.PrivilegeManager;
import cn.com.qmth.examcloud.task.api.DataSyncCloudService;
import cn.com.qmth.examcloud.task.api.request.SyncExamReq;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年8月17日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class ExamServiceImpl implements ExamService {
	private static Pattern pattern = Pattern.compile("\\s*");

	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamStudentService examStudentService;

	@Autowired
	ExamSpecialSettingsRepo examSpecialSettingsRepo;

	@Autowired
	OrgCloudService orgCloudService;

	@Autowired
	ExamPropertyRepo examPropertyRepo;

	@Autowired
	ExamOrgPropertyRepo examOrgPropertyRepo;

	@Autowired
	DataSyncCloudService dataSyncCloudService;

	@Autowired
	ExamSettingsCache examSettingsCache;

	@Autowired
	ExamPropertyCache examPropertyCache;

	private static final String[] EXAM_ORG_SETTINGS_EXCEL_HEADER = new String[]{"学习中心ID", "学习中心代码",
			"学习中心名称", "是否可以考试(是/否)", "开始考试时间 yyyy-MM-dd hh:mm:ss", "结束考试时间 yyyy-MM-dd hh:mm:ss"};

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.examwork.service.ExamService#saveExam(cn.com.
	 * qmth.examcloud.core.examwork.service.bean.ExamInfo)
	 */
	@Override
	public ExamEntity saveExam(ExamInfo examInfo, CURD curd) {

		Long rootOrgId = examInfo.getRootOrgId();
		String code = examInfo.getCode();
		String name = examInfo.getName();
		ExamType examType = examInfo.getExamType();

		if (null == examType) {
			throw new StatusException("001005", "考试类型不能为空");
		}

		if (StringUtils.isBlank(code)) {
			throw new StatusException("001005", "考试编码不能为空");
		}

		if (null == rootOrgId) {
			throw new StatusException("001005", "rootOrgId is null");
		}

		// 上锁
		Object[] locker = new Object[]{"E_EXAM_SAVE_EXAM", rootOrgId, code};
		SequenceLockHelper.getLock(locker);

		GetOrgReq getOrgReq = new GetOrgReq();
		getOrgReq.setOrgId(rootOrgId);
		GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
		OrgBean rootOrg = getOrgResp.getOrg();

		Map<String, String> properties = examInfo.getProperties();
		Map<DynamicEnum, String> map = checkAndGetExamProperties(rootOrgId, properties);

		ExamEntity exam = null;
		CURD realStatus = null;

		// 更新
		if (curd.equals(CURD.UPDATE)) {
			exam = examRepo.findByCodeAndRootOrgId(code, rootOrgId);
			if (null == exam) {
				throw new StatusException("002001", "code is wrong");
			}
			realStatus = CURD.UPDATE;
		}
		// 创建
		else if (curd.equals(CURD.CREATION)) {
			exam = new ExamEntity();
			realStatus = CURD.CREATION;
		}
		// (根据考试编码)新增或创建
		else if (curd.equals(CURD.CREATION_OR_UPDATE)) {
			exam = examRepo.findByCodeAndRootOrgId(code, rootOrgId);
			// 创建
			if (null == exam) {
				exam = new ExamEntity();
				realStatus = CURD.CREATION;
			}
			// 更新
			else {
				realStatus = CURD.UPDATE;
			}
		}

		if (realStatus.equals(CURD.CREATION)) {
			if (null == examInfo.getBeginTime()) {
				throw new StatusException("002006", "beginTime is null");
			}
			if (null == examInfo.getEndTime()) {
				throw new StatusException("002006", "endTime is null");
			}
			if (StringUtils.isBlank(name)) {
				throw new StatusException("002004", "name is blank");
			}

			ExamEntity examByCode = examRepo.findByCodeAndRootOrgId(code, rootOrgId);
			if (null != examByCode) {
				throw new StatusException("002005", "考试编码已存在");
			}

			ExamEntity examByName = examRepo.findByNameAndRootOrgId(name, rootOrgId);
			if (null != examByName) {
				throw new StatusException("002005", "考试名称已存在");
			}

			exam.setCode(code);
			exam.setRootOrgId(rootOrgId);
			exam.setExamType(examType);
			exam.setEnable(true);
			exam.setExamLimit(false);
			exam.setSpecialSettingsEnabled(false);
		} else if (realStatus.equals(CURD.UPDATE)) {
			if (!exam.getRootOrgId().equals(rootOrgId)) {
				throw new StatusException("002003", "rootOrgId is wrong");
			}
			if (!exam.getExamType().equals(examType)) {
				throw new StatusException("002100", "examType is wrong");
			}
		}

		if (null != examInfo.getSpecialSettingsEnabled()) {
			exam.setSpecialSettingsEnabled(examInfo.getSpecialSettingsEnabled());
		}
		if (null != examInfo.getSpecialSettingsType()) {
			exam.setSpecialSettingsType(examInfo.getSpecialSettingsType());
		}
		if (null != examInfo.getBeginTime()) {
			exam.setBeginTime(examInfo.getBeginTime());
		}
		if (null != examInfo.getEndTime()) {
			exam.setEndTime(examInfo.getEndTime());
		}
		if (null != examInfo.getDuration()) {
			exam.setDuration(examInfo.getDuration());
		}
		if (null != examInfo.getEnable()) {
			exam.setEnable(examInfo.getEnable());
		}
		if (null != examInfo.getExamTimes()) {
			exam.setExamTimes(examInfo.getExamTimes());
		}
		if (null != examInfo.getExamLimit()) {
			exam.setExamLimit(examInfo.getExamLimit());
		}
		if (StringUtils.isNotBlank(name)) {
			exam.setName(name);
		}
		if (StringUtils.isNotBlank(examInfo.getRemark())) {
			exam.setName(examInfo.getRemark());
		}

		// 数据订正代码
		exam.setExamLimit(false);

		ExamEntity saved = examRepo.saveAndFlush(exam);

		for (Entry<DynamicEnum, String> entry : map.entrySet()) {
			DynamicEnum de = entry.getKey();
			String value = entry.getValue();
			ExamPropertyEntity entity = examPropertyRepo.findByExamIdAndKeyId(saved.getId(),
					de.getId());
			if (null == entity) {
				entity = new ExamPropertyEntity();
				entity.setExamId(saved.getId());
				entity.setKeyId(de.getId());
			}
			entity.setValue(value);

			examPropertyRepo.save(entity);
		}

		// 考试设置混入特殊设置
		ExamSpecialSettingsInfo examSpecialInfo = new ExamSpecialSettingsInfo();
		examSpecialInfo.setBeginTime(saved.getBeginTime());
		examSpecialInfo.setEndTime(saved.getEndTime());
		examSpecialInfo.setExamId(saved.getId());
		examSpecialInfo.setExamLimit(saved.getExamLimit());
		examSpecialInfo.setRootOrgId(saved.getRootOrgId());
		examSpecialInfo.setExamLimit(saved.getExamLimit());
		this.saveExamSpecialSettings(examSpecialInfo);

		examSpecialSettingsRepo.updateExamEnableByExamId(saved.getId(), saved.getEnable());
		examSpecialSettingsRepo.updateSpecialSettingsEnabledByExamId(saved.getId(),
				saved.getSpecialSettingsEnabled());
		if (null == saved.getSpecialSettingsType()) {
			examSpecialSettingsRepo.updateSpecialSettingsTypeByExamId(saved.getId(), null);
		} else {
			examSpecialSettingsRepo.updateSpecialSettingsTypeByExamId(saved.getId(),
					saved.getSpecialSettingsType());
		}

		SyncExamReq req = new SyncExamReq();
		req.setId(saved.getId());
		req.setBeginTime(saved.getBeginTime());
		req.setDuration(saved.getDuration());
		req.setEnable(saved.getEnable());
		req.setEndTime(saved.getEndTime());
		req.setExamTimes(saved.getExamTimes());
		req.setExamType(saved.getExamType().name());
		req.setName(saved.getName());
		req.setRemark(saved.getRemark());
		req.setRootOrgId(saved.getRootOrgId());
		req.setRootOrgName(rootOrg.getName());
		req.setSyncType("update");
		dataSyncCloudService.syncExam(req);

		examSettingsCache.remove(saved.getId());

		for (String cur : properties.keySet()) {
			examPropertyCache.remove(saved.getId(), cur);
		}

		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param rootOrgId
	 * @param properties
	 * @return
	 */
	private Map<DynamicEnum, String> checkAndGetExamProperties(Long rootOrgId,
			Map<String, String> properties) {
		DynamicEnumManager manager = ExamProperty.getDynamicEnumManager();

		Map<DynamicEnum, String> map = Maps.newHashMap();
		if (null == properties) {
			return map;
		}

		for (Entry<String, String> entry : properties.entrySet()) {
			String key = entry.getKey();
			String value = entry.getValue();
			DynamicEnum de;
			try {
				de = manager.getByName(key);
			} catch (Exception e) {
				throw new StatusException("001004", "考试属性错误. key=" + key);
			}
			if (!de.isLegal(value)) {
				throw new StatusException("001200", "考试属性值非法. key=" + key);
			}
			map.put(de, value);
		}

		String beforeExamRemark = properties.get("BEFORE_EXAM_REMARK");

		if (null != beforeExamRemark) {
			Document doc = Jsoup.parse(beforeExamRemark);
			Matcher m = pattern.matcher(doc.text());
			String simpleText = m.replaceAll("").replaceAll("　", "");
			if (simpleText.length() > 800) {
				throw new StatusException("001002", "考前说明内容不得超过800个字符");
			}
			long imgsize = getImgSizeByBase64String(beforeExamRemark);
			if (imgsize > 150 * 1024) {
				throw new StatusException("001002", "考前说明图片总大小不得超过150KB");
			}
		}

		String afterExamRemark = properties.get("AFTER_EXAM_REMARK");
		if (null != afterExamRemark) {
			Document doc = Jsoup.parse(afterExamRemark);
			Matcher m = pattern.matcher(doc.text());
			String simpleText = m.replaceAll("").replaceAll("　", "");
			if (simpleText.length() > 800) {
				throw new StatusException("001002", "考后说明内容不得超过800个字符");
			}
			long imgsize = getImgSizeByBase64String(afterExamRemark);
			if (imgsize > 150 * 1024) {
				throw new StatusException("001002", "考后说明图片总大小不得超过150KB");
			}
		}

		String cheatingRemark = properties.get("CHEATING_REMARK");
		if (null != cheatingRemark) {
			Document doc = Jsoup.parse(cheatingRemark);
			Matcher m = pattern.matcher(doc.text());
			String simpleText = m.replaceAll("").replaceAll("　", "");
			if (simpleText.length() > 800) {
				throw new StatusException("001002", "作弊说明内容不得超过800个字符");
			}
			long imgsize = getImgSizeByBase64String(cheatingRemark);
			if (imgsize > 150 * 1024) {
				throw new StatusException("001002", "作弊说明图片总大小不得超过150KB");
			}
		}

		// 校验机构权限
		// 人脸识别功能校验
		String faceCheck = PrivilegeDefine.RootOrgFunctions.OnlineExamFunctions.FaceCheck.CODE;
		Boolean hasFaceCheckFunction = PrivilegeManager.judge(rootOrgId, faceCheck);
		String isFaceEnable = properties.get("IS_FACE_ENABLE");
		if (!hasFaceCheckFunction) {
			if (StringUtils.isNotBlank(isFaceEnable)
					&& isFaceEnable.equalsIgnoreCase(Boolean.toString(true))) {
				throw new StatusException("001002", "人脸识别功能未开放");
			}
		}

		// 活体检测功能校验
		String IdentificationOfLivingBody = PrivilegeDefine.RootOrgFunctions.OnlineExamFunctions.IdentificationOfLivingBody.CODE;
		Boolean hasIdentificationOfLivingBodyFunction = PrivilegeManager.judge(rootOrgId,
				IdentificationOfLivingBody);
		String isFceVerify = properties.get("IS_FACE_VERIFY");
		if (!hasIdentificationOfLivingBodyFunction) {
			if (StringUtils.isNotBlank(isFceVerify)
					&& isFceVerify.equalsIgnoreCase(Boolean.toString(true))) {
				throw new StatusException("001002", "活体检测功能未开放");
			}
		}

		return map;
	}

	/**
	 * @param content
	 *            内容
	 * @return 内容中图片文件总大小
	 */
	private long getImgSizeByBase64String(String content) {
		long size = 0;
		if (StringUtils.isNoneEmpty(content)) {
			Document doc = Jsoup.parse(content);
			Elements imgs = doc.getElementsByTag("img");
			if (imgs != null) {
				for (Element img : imgs) {
					String src = img.attr("src");
					if (src.startsWith("data:")) {
						String base64 = src.split(",")[1];
						int equalIndex = base64.indexOf("=");
						if (equalIndex > 0) {
							base64 = base64.substring(0, equalIndex);
						}
						long fileSize = base64.length() - (base64.length() / 8) * 2;
						size = size + fileSize;
					}
				}
			}
		}
		return size;
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.examwork.service.ExamService#saveExamOrg(cn.
	 * com.qmth.examcloud.core.examwork.service.bean.ExamOrgInfo)
	 */
	@Override
	public ExamSpecialSettingsEntity saveExamSpecialSettings(
			ExamSpecialSettingsInfo examSpecialInfo) {

		ExamSpecialSettingsEntity examSpecialSettingsEntity = null;

		Long rootOrgId = examSpecialInfo.getRootOrgId();
		Long examId = examSpecialInfo.getExamId();
		Date beginTime = examSpecialInfo.getBeginTime();
		Date endTime = examSpecialInfo.getEndTime();
		Long orgId = examSpecialInfo.getOrgId();
		Long studentId = examSpecialInfo.getStudentId();
		Long courseId = examSpecialInfo.getCourseId();

		int trueNum = BooleanUtil.countTrue(null != orgId, null != courseId, null != studentId);
		if (1 < trueNum) {
			throw new StatusException("001502", "参数错误");
		}

		// 上锁
		Object[] locker = new Object[]{"E_EXAM_SPECIAL_SETTINGS", examId, "ORG" + orgId,
				"COURSE" + courseId, "Student" + studentId};
		SequenceLockHelper.getLock(locker);

		if (!new Boolean(null == beginTime).equals(new Boolean(null == endTime))) {
			throw new StatusException("001101", "beginTime & endTime  wrong");
		}

		ExamEntity examEntity = GlobalHelper.getPresentEntity(examRepo, examId, ExamEntity.class);
		GlobalHelper.uniformRootOrg(examEntity.getRootOrgId(), rootOrgId);

		StudentCacheBean student = null;
		if (null != studentId) {
			student = CacheHelper.getStudent(studentId);
			GlobalHelper.uniformRootOrg(student.getRootOrgId(), rootOrgId);
		}

		if (null == examSpecialInfo.getId()) {
			// 机构特殊设置
			if (null != orgId && null == courseId && null == studentId) {
				examSpecialSettingsEntity = examSpecialSettingsRepo
						.findByExamIdAndOrgIdAndCourseIdIsNullAndStudentIdIsNull(
								examSpecialInfo.getExamId(), orgId);
			}
			// 学生特殊设置
			else if (null == orgId && null == courseId && null != studentId) {
				examSpecialSettingsEntity = examSpecialSettingsRepo
						.findByExamIdAndStudentIdAndOrgIdIsNullAndCourseIdIsNull(
								examSpecialInfo.getExamId(), studentId);
			}
			// 课程特殊设置
			else if (null == orgId && null != courseId && null == studentId) {

			}
			// 考试设置
			else if (null == orgId && null == courseId && null == studentId) {
				examSpecialSettingsEntity = examSpecialSettingsRepo
						.findAllByExamIdAndOrgIdIsNullAndCourseIdIsNullAndStudentIdIsNull(
								examSpecialInfo.getExamId());
			} else {
				throw new StatusException("001501", "参数错误");
			}

			if (null == examSpecialSettingsEntity) {
				examSpecialSettingsEntity = new ExamSpecialSettingsEntity();
			}

			examSpecialSettingsEntity.setRootOrgId(rootOrgId);
			examSpecialSettingsEntity.setExamId(examId);
			examSpecialSettingsEntity.setExamType(examEntity.getExamType());
			examSpecialSettingsEntity.setExamEnable(examEntity.getEnable());
			examSpecialSettingsEntity
					.setSpecialSettingsEnabled(examEntity.getSpecialSettingsEnabled());
			examSpecialSettingsEntity.setSpecialSettingsType(examEntity.getSpecialSettingsType());

			examSpecialSettingsEntity.setOrgId(orgId);
			examSpecialSettingsEntity.setCourseId(courseId);
			examSpecialSettingsEntity.setStudentId(studentId);

			// 机构特殊设置
			if (null != orgId && null == courseId && null == studentId) {
			}
			// 学生特殊设置
			else if (null == orgId && null == courseId && null != studentId) {
				if (null != studentId) {
					examSpecialSettingsEntity.setExt1(student.getIdentityNumber());
				}
			}
			// 课程特殊设置
			else if (null == orgId && null != courseId && null == studentId) {
			}
			// 考试设置
			else if (null == orgId && null == courseId && null == studentId) {
			}

		} else {
			examSpecialSettingsEntity = GlobalHelper.getEntity(examSpecialSettingsRepo,
					examSpecialInfo.getId(), ExamSpecialSettingsEntity.class);
			if (null == examSpecialSettingsEntity) {
				throw new StatusException("001101", "id is wrong");
			}
		}

		examSpecialSettingsEntity.setBeginTime(examSpecialInfo.getBeginTime());
		examSpecialSettingsEntity.setEndTime(examSpecialInfo.getEndTime());

		Boolean examLimit = examSpecialInfo.getExamLimit();
		examLimit = null == examLimit ? false : examLimit;
		examSpecialSettingsEntity.setExamLimit(examLimit);

		ExamSpecialSettingsEntity saved = examSpecialSettingsRepo.save(examSpecialSettingsEntity);

		Map<String, String> props = examSpecialInfo.getProperties();

		if (null != props) {
			Map<DynamicEnum, String> map = checkAndGetExamProperties(examEntity.getRootOrgId(),
					props);

			for (Entry<DynamicEnum, String> entry : map.entrySet()) {
				DynamicEnum de = entry.getKey();
				String value = entry.getValue();
				if (StringUtils.isBlank(value)) {
					value = null;
				} else {
					value = value.trim();
				}

				// 机构特殊设置
				if (null != orgId && null == courseId && null == studentId) {
					ExamOrgPropertyEntity entity = examOrgPropertyRepo.findByExamIdAndOrgIdAndKeyId(
							saved.getExamId(), saved.getOrgId(), de.getId());
					if (null == entity) {
						entity = new ExamOrgPropertyEntity();
						entity.setExamId(saved.getExamId());
						entity.setOrgId(saved.getOrgId());
						entity.setKeyId(de.getId());
					}
					entity.setValue(value);

					examOrgPropertyRepo.save(entity);
				}
				// 学生特殊设置
				else if (null == orgId && null == courseId && null != studentId) {

				}
				// 课程特殊设置
				else if (null == orgId && null != courseId && null == studentId) {

				}

			}
		}

		return saved;
	}

	@Override
	public String getExamOrgProperty(Long examId, Long orgId, String key) {
		DynamicEnumManager manager = ExamProperty.getDynamicEnumManager();
		DynamicEnum de = manager.getByName(key);

		if (null != orgId) {
			ExamOrgPropertyEntity examOrgPropertyEntity = examOrgPropertyRepo
					.findByExamIdAndOrgIdAndKeyId(examId, orgId, de.getId());
			if (null != examOrgPropertyEntity) {
				return examOrgPropertyEntity.getValue();
			}
		}

		ExamPropertyEntity examPropertyEntity = examPropertyRepo.findByExamIdAndKeyId(examId,
				de.getId());
		if (null == examPropertyEntity) {
			return null;
		}

		return examPropertyEntity.getValue();
	}

	@Override
	public String getExamStudentProperty(Long examId, Long studentId, String key) {
		DynamicEnumManager manager = ExamProperty.getDynamicEnumManager();
		DynamicEnum de = manager.getByName(key);

		if (null != studentId) {
			// 待实现
		}

		ExamPropertyEntity examPropertyEntity = examPropertyRepo.findByExamIdAndKeyId(examId,
				de.getId());
		if (null == examPropertyEntity) {
			return null;
		}

		return examPropertyEntity.getValue();
	}

	@Override
	public List<Map<String, Object>> importExamOrgSettings(Long examId, File file) {

		ExamEntity examEntity = GlobalHelper.getEntity(examRepo, examId, ExamEntity.class);
		List<String[]> lineList = null;
		try {
			lineList = ExcelReader.readSheetBySax(PathUtil.getCanonicalPath(file), 1, 6);
		} catch (Exception e) {
			throw new StatusException("200110", "Excel 解析失败");
		}

		if (CollectionUtils.isEmpty(lineList)) {
			throw new StatusException("200111", "Excel无内容");
		}

		if (10001 < lineList.size()) {
			throw new StatusException("200112", "数据行数不能超过10000");
		}

		List<Map<String, Object>> failRecords = Collections
				.synchronizedList(new ArrayList<Map<String, Object>>());
		List<ExamSpecialSettingsInfo> specialSettingsList = Lists.newArrayList();

		for (int i = 0; i < lineList.size(); i++) {
			String[] line = lineList.get(i);
			if (0 == i) {
				if (headerError(line)) {
					throw new StatusException("100111", "Excel表头错误");
				}
				continue;
			}

			boolean hasError = false;
			StringBuilder msg = new StringBuilder();

			ExamSpecialSettingsInfo examSpecialInfo = new ExamSpecialSettingsInfo();
			examSpecialInfo.setRootOrgId(examEntity.getRootOrgId());
			examSpecialInfo.setExamId(examId);

			String orgId = trimAndNullIfBlank(line[0]);
			if (StringUtils.isBlank(orgId)) {
				msg.append("  学习中心ID不能为空");
				hasError = true;
			} else {
				try {
					long orgIdLong = Long.parseLong(orgId);

					GetOrgReq getOrgReq = new GetOrgReq();
					getOrgReq.setOrgId(orgIdLong);
					GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
					OrgBean orgBean = getOrgResp.getOrg();

					if (null == orgBean.getParentId()
							|| !orgBean.getRootId().equals(examEntity.getRootOrgId())) {
						msg.append("  学习中心ID非法");
						hasError = true;
					} else {
						examSpecialInfo.setOrgId(orgIdLong);
					}
				} catch (NumberFormatException e) {
					msg.append("  学习中心ID必须为整数");
					hasError = true;
				}
			}

			String examLimit = trimAndNullIfBlank(line[3]);
			if (StringUtils.isNotBlank(examLimit)) {
				if (examLimit.equals("是")) {
					examSpecialInfo.setExamLimit(false);
				} else if (examLimit.equals("否")) {
					examSpecialInfo.setExamLimit(true);
				} else {
					msg.append("  是否可以开始考试必须为['是','否',空]");
					hasError = true;
				}
			} else {
				examSpecialInfo.setExamLimit(false);
			}

			String beginTime = trimAndNullIfBlank(line[4]);
			if (StringUtils.isNotBlank(beginTime)) {
				try {
					Date beginDate = DateUtil.parse(beginTime, DatePatterns.CHINA_DEFAULT);
					examSpecialInfo.setBeginTime(beginDate);
				} catch (Exception e) {
					msg.append("  开始考试时间格式错误. 正确格式为 " + DatePatterns.CHINA_DEFAULT);
					hasError = true;
				}
			}
			String endTime = trimAndNullIfBlank(line[5]);
			if (StringUtils.isNotBlank(endTime)) {
				try {
					Date endDate = DateUtil.parse(endTime, DatePatterns.CHINA_DEFAULT);
					examSpecialInfo.setEndTime(endDate);
				} catch (Exception e) {
					msg.append("  结束考试时间格式错误. 正确格式为 " + DatePatterns.CHINA_DEFAULT);
					hasError = true;
				}
			}

			if (hasError) {
				failRecords.add(newError(i + 1, msg.toString()));
			} else {
				specialSettingsList.add(examSpecialInfo);
			}

		}

		if (CollectionUtils.isNotEmpty(failRecords)) {
			return failRecords;
		}

		for (ExamSpecialSettingsInfo cur : specialSettingsList) {
			saveExamSpecialSettings(cur);
		}

		return failRecords;

	}

	private Map<String, Object> newError(int lineNum, String msg) {
		Map<String, Object> map = Maps.newHashMap();
		map.put("lineNum", lineNum);
		map.put("msg", msg);
		return map;
	}

	private String trimAndNullIfBlank(String s) {
		if (StringUtils.isBlank(s)) {
			return null;
		}
		return s.trim();
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param header
	 */
	private boolean headerError(String[] header) {
		for (int i = 0; i < EXAM_ORG_SETTINGS_EXCEL_HEADER.length; i++) {
			if (null == header[i]) {
				return true;
			}
			if (!EXAM_ORG_SETTINGS_EXCEL_HEADER[i].equals(header[i].trim())) {
				return true;
			}
		}
		return false;
	}

	@Override
	public String getExamProperty(Long examId, String key) {
		DynamicEnumManager manager = ExamProperty.getDynamicEnumManager();
		DynamicEnum de = manager.getByName(key);

		ExamPropertyEntity examPropertyEntity = examPropertyRepo.findByExamIdAndKeyId(examId,
				de.getId());
		if (null == examPropertyEntity) {
			return null;
		}

		return examPropertyEntity.getValue();
	}

}
