package cn.com.qmth.examcloud.core.examwork.starter.config;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import cn.com.qmth.examcloud.web.interceptor.ApiFlowLimitedInterceptor;
import cn.com.qmth.examcloud.web.interceptor.ApiStatisticInterceptor;
import cn.com.qmth.examcloud.web.interceptor.FirstInterceptor;
import cn.com.qmth.examcloud.web.interceptor.SeqlockInterceptor;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.security.RequestPermissionInterceptor;
import cn.com.qmth.examcloud.web.security.ResourceManager;
import cn.com.qmth.examcloud.web.security.RpcInterceptor;

/**
 * WebMvcConfigurer
 *
 * @author WANGWEI
 * @date 2019年1月30日
 * @Copyright (c) 2018-2020 http://www.qmth.com.cn/ All Rights Reserved.
 */
@Configuration
public class ExamCloudWebMvcConfigurer implements WebMvcConfigurer {

	@Autowired
	ResourceManager resourceManager;

	@Autowired
	RedisClient redisClient;

	@Override
	public void addInterceptors(InterceptorRegistry registry) {
		registry.addInterceptor(new FirstInterceptor()).addPathPatterns("/**");
		registry.addInterceptor(new ApiFlowLimitedInterceptor()).addPathPatterns("/**");
		registry.addInterceptor(new RpcInterceptor(resourceManager)).addPathPatterns("/**");

		RequestPermissionInterceptor permissionInterceptor = new RequestPermissionInterceptor(
				resourceManager, redisClient);
		registry.addInterceptor(permissionInterceptor).addPathPatterns("/**");
		registry.addInterceptor(new SeqlockInterceptor(redisClient)).addPathPatterns("/**");
		registry.addInterceptor(new ApiStatisticInterceptor()).addPathPatterns("/**");
	}

	@Override
	public void addCorsMappings(CorsRegistry registry) {
		registry.addMapping("/**").allowedOrigins("*").allowCredentials(false).allowedMethods("*")
				.maxAge(3600);
	}

}